/**
 * Copyright (c) 2014 - 2020, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup ble_sdk_uart_over_ble_main main.c
 * @{
 * @ingroup  ble_sdk_app_nus_eval
 * @brief    UART over BLE application main file.
 *
 * This file contains the source code for a sample application that uses the Nordic UART service.
 * This application uses the @ref srvlib_conn_params module.
 */




/******************************************************************************************************************
*
*	V2022/01/25
*	@ Battery 연결시 부저음 추가 
*	@ 와치독 적용
*
*	
*  V2022/02/10
*  @ 비상벨 AES 암호화
*	
*
*
*	
*
*
*	
*
*
*	
*
*
*
*
*	
*
*
*	
*
*
*******************************************************************************************************************/	

























#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_pwm.h"

#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_delay.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_drv_saadc.h"
#include "nrf_drv_rtc.h"
#include "nrf_drv_wdt.h"


#include "main.h"
#include "sysmgr.h"
#include "dio.h"
#include "nrf_power.h"

#include "fds.h"
#include "nrf_fstorage.h"


#define APP_BLE_CONN_CFG_TAG            1                                           /**< A tag identifying the SoftDevice BLE configuration. */

//#define DEVICE_NAME                     "Nordic_UART"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_BLE_OBSERVER_PRIO           3                                           /**< Application's BLE observer priority. You shouldn't need to modify this value. */

#define APP_ADV_INTERVAL                64                                          /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */

#define APP_ADV_DURATION                300                                       /**< The advertising duration (180 seconds) in units of 10 milliseconds. */

#define APP_TIMER_PRESCALER             0 



#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */

#define COMPARE_COUNTERTIME  (3UL)                                        /**< Get Compare event COMPARE_TIME seconds after the counter starts from 0. */
#define TICK_EVENT_OUTPUT     BSP_LED_0                                 /**< Pin number for indicating tick event. */
#define COMPARE_EVENT_OUTPUT   BSP_LED_1

BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);                                   /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                             /**< Context for the Queued Write module.*/
BLE_ADVERTISING_DEF(m_advertising);                                                 /**< Advertising module instance. */


#define TIMER_MS_INTERVAL        			 APP_TIMER_TICKS(25)



APP_TIMER_DEF(m_ms_timer_id);
APP_TIMER_DEF(wakeup_timer_id);
APP_TIMER_DEF(m_our_char_timer_id);
nrf_drv_wdt_channel_id m_channel_id;


#define KEEPALIVE_INTERVAL_30_SEC	30*1000
#define KEEPALIVE_INTERVAL_1_MIN	60*1000
#define KEEPALIVE_INTERVAL_30_MIN	30*60*1000
#define KEEPALIVE_INTERVAL_1_HOUR	60*60*1000
#define KEEPALIVE_INTERVAL_1_DAY	24*60*6*1000



#define WAKEUP_TIME_INVERVAL	APP_TIMER_TICKS(KEEPALIVE_INTERVAL_1_HOUR)
//#define WAKEUP_TIME_INVERVAL	APP_TIMER_TICKS(KEEPALIVE_INTERVAL_30_SEC)

//#define WAKEUP_TIME_INVERVAL	APP_TIMER_TICKS(30*1000)
//#define WAKEUP_TIME_INVERVAL	APP_TIMER_TICKS(15000)


#define OUR_CHAR_TIMER_INTERVAL APP_TIMER_TICKS(1) // 1 ms intervals


char DEVICE_NAME[] =   {'I', 'C', 'S', 'W', 'E', 'S', 0x44,0x33,0xC0,0xFF,0xFF,0xFF };
//char DEVICE_NAME[] =   {'I', 'C', 'S', 'P', 'C', 'A', 0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff };


/* Flag to check fds initialization. */
static bool volatile m_fds_initialized;
/* Array to map FDS events to strings. */
static char const * fds_evt_str[] =
{
    "FDS_EVT_INIT",
    "FDS_EVT_WRITE",
    "FDS_EVT_UPDATE",
    "FDS_EVT_DEL_RECORD",
    "FDS_EVT_DEL_FILE",
    "FDS_EVT_GC",
};


/* Keep track of the progress of a delete_all operation. */
static struct
{
    bool delete_next;   //!< Delete next record.
    bool pending;       //!< Waiting for an fds FDS_EVT_DEL_RECORD event, to delete the next record.
} m_delete_all;





static void uart_init(void);
static void advertising_init(void);
static void buttons_leds_init(bool * p_erase_bonds);
static void log_init(void);
static void power_management_init(void);
void wakeup_timer_handler(void * p_context);

static void leds_config(void);

void Battery_Adc_Read(void);



static void idle_state_handle(void);
//static void advertising_start(void);
static void timer_ms_timeout_handler(void * p_context);
static void on_adv_evt(ble_adv_evt_t ble_adv_evt);





uint8_t advertising_flag = 0;






static void fds_evt_handler(fds_evt_t const * p_evt)
{
    if (p_evt->result == NRF_SUCCESS)
    {
        NRF_LOG_INFO("Event: %s received (NRF_SUCCESS)",
                      fds_evt_str[p_evt->id]);
    }
    else
    {
        NRF_LOG_INFO("Event: %s received (%s)",
                      fds_evt_str[p_evt->id],
                      fds_err_str(p_evt->result));
    }

    switch (p_evt->id)
    {
        case FDS_EVT_INIT:
            if (p_evt->result == NRF_SUCCESS)
            {
                m_fds_initialized = true;
            }
            break;

        case FDS_EVT_WRITE:
        {
            if (p_evt->result == NRF_SUCCESS)
            {
                NRF_LOG_INFO("Record ID:\t0x%04x",  p_evt->write.record_id);
                NRF_LOG_INFO("File ID:\t0x%04x",    p_evt->write.file_id);
                NRF_LOG_INFO("Record key:\t0x%04x", p_evt->write.record_key);
            }
        } break;

        case FDS_EVT_DEL_RECORD:
        {
            if (p_evt->result == NRF_SUCCESS)
            {
                NRF_LOG_INFO("Record ID:\t0x%04x",  p_evt->del.record_id);
                NRF_LOG_INFO("File ID:\t0x%04x",    p_evt->del.file_id);
                NRF_LOG_INFO("Record key:\t0x%04x", p_evt->del.record_key);
            }
            m_delete_all.pending = false;
        } break;

        default:
            break;
    }
}




void ReMac_Mapping();
ble_gap_addr_t old_ble_addr;
ble_gap_addr_t new_ble_addr;


int SysStatus;
int SysCounter;

static uint16_t   m_conn_handle          = BLE_CONN_HANDLE_INVALID;                 /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
static ble_uuid_t m_adv_uuids[]          =                                          /**< Universally unique service identifier. */
{
    {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};
APP_PWM_INSTANCE(PWM1,1);                   // Create the instance "PWM1" using TIMER1.

static volatile bool ready_flag;            // A flag indicating PWM status.
#define SAMPLES_IN_BUFFER 5

static nrf_saadc_value_t     m_buffer_pool[SAMPLES_IN_BUFFER];

int SysCounter;


ble_advertising_t* p_m_advertising;



void SendEmgData(void);
void SendTestData(void);
void SendRegData(void);
void SendAliveData(void);
void bsp_event_handler(bsp_event_t event);



void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
    {
      nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAMPLES_IN_BUFFER);
      
    }
    
    for(uint8_t i = 1 ; i < SAMPLES_IN_BUFFER ; i++)
    {
      NRF_LOG_INFO("ADC:%04d\r\n",p_event->data.done.p_buffer[i]);
    }
}

void SaadcInit(void)
{

  nrf_saadc_channel_config_t channel_config =
      NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN2);
  nrf_drv_saadc_init(NULL, saadc_callback);

  nrf_drv_saadc_channel_init(2, &channel_config);
  

}


void wdt_event_handler(void)
{
    //bsp_board_leds_off();
}


void BootLoad_Reboot()
{

	nrf_power_gpregret_set(0xB1);
	NVIC_SystemReset();

}





// ALREADY_DONE_FOR_YOU: This is a timer event handler
// 1m sec Interrupt
static void timer_timeout_handler(void * p_context)
{
	uint32_t err_code;
    SystemTimer.MS_1++;
    SystemTimer.UART_LAST_RECV_TIMER++;
    SystemTimer.SEND_TIMER++;
    SystemTimer.BUZZER_TIMER--;

	SystemManager.BleConnectTime++;

	SystemTimer.TIM_KEY++;
	SystemTimer.SLEEP_TIMER++;


	// 500ms 마다 SystemManager.BleSendStatus 상태확인하여 수신기에 데이타 전송

	if(SystemManager.BleConnected){

		if( SystemManager.BleSendStatus &&  (SystemTimer.SEND_TIMER%500 ==0))
		{
			NRF_LOG_INFO("SystemManager.BleConnected:%d\n", SystemManager.BleConnected);

			if( SystemManager.EmgSendButton == BUTTON_EMG )			// EMG
			{
				SendEmgData();
				SystemTimer.EMG_SEND_RUN = 0;

			}else if( SystemManager.EmgSendButton == BUTTON_TEST )		// TEST
			{
				SendTestData(); 
				SystemTimer.EMG_SEND_RUN = 0;
				//SystemManager.EmgSendButton = BUTTON_NONE;

			}else if( SystemManager.EmgSendButton == BUTTON_ALIVE )		// Alive
			{
				SendAliveData(); 
				SystemTimer.EMG_SEND_RUN = 0;
				//SystemManager.EmgSendButton = BUTTON_NONE;

			}else if( SystemManager.EmgSendButton == BUTTON_REG )		// 등록
			{
				SendRegData(); 
				SystemTimer.EMG_SEND_RUN = 0;
				//SystemManager.EmgSendButton = 11;
				//ParkPowerOff();
			}			  
		}
	}else{

		
		//NRF_LOG_INFO("SystemManager.BleConnected:%d\n", SystemManager.BleConnected);
		// BLE ADVERTISE RECONNECT START
	}


	if( SystemTimer.SEND_TIMER > 3000 )
	{
		NVIC_SystemReset();		// SYSTEM RESET
	}


#if 0
	if( SystemManager.EmgSendButton == 3 || SystemManager.EmgSendButton == 11 )
	{
		if( SystemManager.BleConnectTime > 6000 )
		{
			NVIC_SystemReset();
			//sd_ble_gap_adv_stop(m_advertising.adv_handle); 
			///m_conn_handle = BLE_CONN_HANDLE_INVALID;
			//sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_LOCAL_HOST_TERMINATED_CONNECTION);
			//sd_ble_gap_adv_stop(m_advertising.adv_handle);  

		}
	}else{

		if( SystemManager.BleConnectTime > 3000 )
		{
			NVIC_SystemReset();
			//sd_ble_gap_adv_stop(m_advertising.adv_handle); 
			///m_conn_handle = BLE_CONN_HANDLE_INVALID;
			//sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_LOCAL_HOST_TERMINATED_CONNECTION);
			//sd_ble_gap_adv_stop(m_advertising.adv_handle);  

		}
	}
#endif



#if 1

	switch(SystemManager.BuzzerType){
		
		case BUZZER_EMG:	// EMG

			if(SystemManager.Batt > BATTERY_LOW_LEVEL){

		
				if( SystemTimer.BUZZER_TIMER == 150 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 100 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}else if( SystemTimer.BUZZER_TIMER < 50 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);

					SystemManager.BuzzerType = BUZZER_NONE;

					app_pwm_disable(&PWM1);
				}

			}else{

				if( SystemTimer.BUZZER_TIMER == 500 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 450 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}else if( SystemTimer.BUZZER_TIMER == 350 )
							{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 300 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}else if( SystemTimer.BUZZER_TIMER == 200 )
							{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 150 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				
				}else if( SystemTimer.BUZZER_TIMER < 100 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
					SystemManager.BuzzerType = BUZZER_NONE;
					app_pwm_disable(&PWM1);
				}
			}
		break;

		case BUZZER_TEST:	//TEST
			if(SystemManager.Batt > BATTERY_LOW_LEVEL){

				if( SystemTimer.BUZZER_TIMER < 10 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
					SystemManager.BuzzerType = BUZZER_NONE;
					app_pwm_disable(&PWM1);
				}
			}else{		// LOW BATTERY (20% Under)
				if( SystemTimer.BUZZER_TIMER == 500 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 450 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}else if( SystemTimer.BUZZER_TIMER == 350 )
	                        {
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 300 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}else if( SystemTimer.BUZZER_TIMER == 200 )
	                        {
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 150 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);

				}else if( SystemTimer.BUZZER_TIMER < 100 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
					SystemManager.BuzzerType = BUZZER_NONE;
					app_pwm_disable(&PWM1);
				}
			}
		break;
			
		case BUZZER_REG:	//등록

			if( SystemTimer.BUZZER_TIMER < 100 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
			}else if( SystemTimer.BUZZER_TIMER == 100 )
			{
			//	app_pwm_channel_duty_set(&PWM1, 0, 50);
			}else if( SystemTimer.BUZZER_TIMER < 50 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
				SystemManager.BuzzerType = BUZZER_NONE;
				app_pwm_disable(&PWM1);
			}
			
		break;


		case BUZZER_BOOT_LOADER: //BOOT LOAD REBOOT

			if( SystemTimer.BUZZER_TIMER == 800 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
			}else if( SystemTimer.BUZZER_TIMER == 700 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 50);
			}else if( SystemTimer.BUZZER_TIMER == 500 )
						{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
			}else if( SystemTimer.BUZZER_TIMER == 400 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 50);
			}else if( SystemTimer.BUZZER_TIMER == 200 )
						{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
			}else if( SystemTimer.BUZZER_TIMER == 100 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 50);

			}else if( SystemTimer.BUZZER_TIMER < 100 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
				SystemManager.BuzzerType = BUZZER_NONE;
				app_pwm_disable(&PWM1);

				BootLoad_Reboot();
			}

		break;



		case BUZZER_POWER_ON:	// 밧데리 연결시 동작	

				if( SystemTimer.BUZZER_TIMER == 250 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 200 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}
				else if( SystemTimer.BUZZER_TIMER == 150 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);
				}else if( SystemTimer.BUZZER_TIMER == 100 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 50);
				}else if( SystemTimer.BUZZER_TIMER < 50 )
				{
					app_pwm_channel_duty_set(&PWM1, 0, 0);

					SystemManager.BuzzerType = BUZZER_NONE;

					app_pwm_disable(&PWM1);
					SystemManager.GoSleep = 1;
				}		
		break;				

#if 0
		case 10:
			if( SystemTimer.BUZZER_TIMER == 150 )
			{
				//app_pwm_channel_duty_set(&PWM1, 0, 50);
			}else if( SystemTimer.BUZZER_TIMER == 100 )
			{
				//app_pwm_channel_duty_set(&PWM1, 0, 50);
			}else if( SystemTimer.BUZZER_TIMER < 50 )
			{
				app_pwm_channel_duty_set(&PWM1, 0, 0);
				SystemManager.BuzzerType = 100;
				app_pwm_disable(&PWM1);
			}
		break;
#endif			
	}

	
#endif


}



/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

/**@brief Function for initializing the timer module.
 */
static void timers_init(void)
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);

    app_timer_create(&m_our_char_timer_id, APP_TIMER_MODE_REPEATED, timer_timeout_handler);

	app_timer_create(&m_ms_timer_id, APP_TIMER_MODE_REPEATED, timer_ms_timeout_handler);
}

/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_evt       Nordic UART Service event.
 */
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_evt_t * p_evt)
{

	uint32_t i;

	uint32_t err_code;

	uint8_t Aes_temp[16];
	uint8_t Aes_Dec_temp[16];


	switch(p_evt->type){
		case BLE_NUS_EVT_RX_DATA:
		
			NRF_LOG_RAW_INFO("BLE_NUS_EVT_RX_DATA (AES) => ");


			for (i = 0; i < p_evt->params.rx_data.length; i++)
	        {
	            NRF_LOG_INFO("0x%02x ",  *(p_evt->params.rx_data.p_data+i));
	        }
			NRF_LOG_INFO("\n");

			for(i=0; i<16; i++){
			
				Aes_temp[i] = *(p_evt->params.rx_data.p_data+i);
			}
			
			WesDecryptEcb( &Aes_temp[0], &Aes_Dec_temp[0] );
			
			NRF_LOG_RAW_INFO("BLE_NUS_EVT_RX_DATA (ORG) => ");
			
			for (i=0; i<16; i++)
			{
				NRF_LOG_RAW_INFO("0x%02x, ", Aes_Dec_temp[i]);
			}
			NRF_LOG_RAW_INFO("\n");

		break;

		case BLE_NUS_EVT_TX_RDY:

			NRF_LOG_RAW_INFO("BLE_NUS_EVT_TX_RDY ======> \n");

		break;

		case BLE_NUS_EVT_COMM_STARTED:
			NRF_LOG_RAW_INFO("BLE_NUS_EVT_COMM_STARTED ======> \n");
		break;
		
		case BLE_NUS_EVT_COMM_STOPPED:
			NRF_LOG_RAW_INFO("BLE_NUS_EVT_COMM_STOPPED ======> \n");
		break;


    }

}
/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t           err_code;
    ble_nus_init_t     nus_init;
    nrf_ble_qwr_init_t qwr_init = {0};

    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);

    // Initialize NUS.
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
    uint32_t err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);
    SysStatus = 300;
    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            NRF_LOG_INFO("BLE_ADV_EVT_FAST");
			SystemManager.BleConnectTime = 0;
            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            SysStatus = 200;
            APP_ERROR_CHECK(err_code);
        break;
		
        case BLE_ADV_EVT_IDLE:
            NRF_LOG_INFO("BLE_ADV_EVT_IDLE");
			err_code = bsp_indication_set(BSP_INDICATE_IDLE);
            APP_ERROR_CHECK(err_code);

			app_pwm_disable(&PWM1);

			app_timer_stop(m_ms_timer_id);
			app_timer_stop(m_our_char_timer_id);

			NVIC_SystemReset();

            app_timer_start(wakeup_timer_id, WAKEUP_TIME_INVERVAL, NULL);
            err_code = sd_app_evt_wait();
            APP_ERROR_CHECK(err_code);

			//StartRTC();
			//app_timer_start(wakeup_timer_id, APP_TIMER_TICKS(15000), NULL);

			        // Enter System ON sleep mode
            //__WFE();
            // Make sure any pending events are cleared
            //__SEV();
           // __WFE();

			
            //sleep_mode_enter();
           // err_code = bsp_indication_set(BSP_INDICATE_IDLE);
            //APP_ERROR_CHECK(err_code);
            //app_timer_start(wakeup_timer_id, APP_TIMER_TICKS(5000), NULL);
			// err_code = sd_app_evt_wait();
            //SysStatus = 201;
            //app_uart_close();
            //idle_state_handle();
            break;
        default:
            break;
    }
}



int localNameChg = 0;


/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {


        case BLE_GAP_EVT_ADV_REPORT :
            NRF_LOG_INFO("BLE_GAP_EVT_ADV_REPORT ");
        break;


		case BLE_GAP_EVT_SCAN_REQ_REPORT:

			 NRF_LOG_INFO("BLE_GAP_EVT_SCAN_REQ_REPORT");

			ble_gap_evt_scan_req_report_t * p_scan_req_report_t = (ble_gap_evt_scan_req_report_t *)&p_ble_evt->evt.gap_evt.params.scan_req_report;

			NRF_LOG_INFO("Peer Address = 0x%02x:%02x:%02x:%02x:%02x:%02x",
					   p_scan_req_report_t->peer_addr.addr[0],
					   p_scan_req_report_t->peer_addr.addr[1],
					   p_scan_req_report_t->peer_addr.addr[2],
					   p_scan_req_report_t->peer_addr.addr[3],
					   p_scan_req_report_t->peer_addr.addr[4],
					   p_scan_req_report_t->peer_addr.addr[5]);

			NRF_LOG_INFO("RSSI value = %d dBm", p_scan_req_report_t->rssi);
			
			SystemManager.PeerAddr[0] = p_scan_req_report_t->peer_addr.addr[0];
			SystemManager.PeerAddr[1] = p_scan_req_report_t->peer_addr.addr[1];
			SystemManager.PeerAddr[2] = p_scan_req_report_t->peer_addr.addr[2];
			SystemManager.PeerAddr[3] = p_scan_req_report_t->peer_addr.addr[3];
			SystemManager.PeerAddr[4] = p_scan_req_report_t->peer_addr.addr[4];
			SystemManager.PeerAddr[5] = p_scan_req_report_t->peer_addr.addr[5];
			
			sprintf(SystemManager.StrPeerAddr, "%02x%02x%02x%02x%02x%02x", p_scan_req_report_t->peer_addr.addr[0],
				  p_scan_req_report_t->peer_addr.addr[1],
				  p_scan_req_report_t->peer_addr.addr[2],
				  p_scan_req_report_t->peer_addr.addr[3],
				  p_scan_req_report_t->peer_addr.addr[4],
				  p_scan_req_report_t->peer_addr.addr[5]);
			SystemManager.StrPeerAddr[12] = 0;
			
			if( Key1Flag )
				break;

			if( strncmp( SystemManager.StrRegPeerAddr, SystemManager.StrPeerAddr, 12 ) != 0 )
			{
				break;
			}

			NRF_LOG_INFO("Reg Receiver Arrived.\n");

			if( localNameChg++ == 2 )
			{

				ble_gap_conn_params_t   gap_conn_params;
				ble_gap_conn_sec_mode_t sec_mode;

				memcpy( &DEVICE_NAME[6], SystemManager.RegPeerAddr, 6);

				BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
				sd_ble_gap_device_name_set(&sec_mode, (const uint8_t *)DEVICE_NAME, strlen(DEVICE_NAME));
				
				sd_ble_gap_adv_stop(p_m_advertising->adv_handle); 

				memcpy( &p_m_advertising->adv_data.adv_data.p_data[14], &DEVICE_NAME[6], 6 );
				memcpy( &p_m_advertising->adv_data.scan_rsp_data.p_data[14], &DEVICE_NAME[6], 6 );

				//ble_advdata_encode(p_m_advertising->p_adv_data, p_m_advertising->adv_data.adv_data.p_data, &p_m_advertising->adv_data.adv_data.len);
				sd_ble_gap_adv_set_configure(&p_m_advertising->adv_handle, &p_m_advertising->adv_data, &p_m_advertising->adv_params);
				
				sd_ble_gap_adv_start(p_m_advertising->adv_handle, APP_BLE_CONN_CFG_TAG);

				NRF_LOG_INFO("Local Name Change = %d", err_code);
				
			}
			else if( localNameChg == 2)
			{

			}
			
		break;


        case BLE_GAP_EVT_CONNECTED:
			
			NRF_LOG_INFO("<============ BLE_GAP_EVT_CONNECTED =========>");
			
			err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
			APP_ERROR_CHECK(err_code);
			m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
			err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
			APP_ERROR_CHECK(err_code);

			SystemManager.BleConnected = 1;
			SystemTimer.SEND_TIMER = 0;

			SystemManager.PeerAddr[0] = p_ble_evt->evt.gap_evt.params.connected.peer_addr.addr[0];
			SystemManager.PeerAddr[1] = p_ble_evt->evt.gap_evt.params.connected.peer_addr.addr[1];
			SystemManager.PeerAddr[2] = p_ble_evt->evt.gap_evt.params.connected.peer_addr.addr[2];
			SystemManager.PeerAddr[3] = p_ble_evt->evt.gap_evt.params.connected.peer_addr.addr[3];
			SystemManager.PeerAddr[4] = p_ble_evt->evt.gap_evt.params.connected.peer_addr.addr[4];
			SystemManager.PeerAddr[5] = p_ble_evt->evt.gap_evt.params.connected.peer_addr.addr[5];

			sprintf(SystemManager.StrPeerAddr, "%02x%02x%02x%02x%02x%02x", SystemManager.PeerAddr[0],
				SystemManager.PeerAddr[1],
				SystemManager.PeerAddr[2],
				SystemManager.PeerAddr[3],
				SystemManager.PeerAddr[4],
				SystemManager.PeerAddr[5]);
			SystemManager.StrPeerAddr[12] = 0;

		break;

        case BLE_GAP_EVT_DISCONNECTED:
			
			NRF_LOG_INFO("<============ BLE_GAP_EVT_DISCONNECTED =========>");

            // LED indication will be changed when advertising starts.
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
			NVIC_SystemReset();

#if 0
			err_code = bsp_indication_set(BSP_INDICATE_IDLE);
            APP_ERROR_CHECK(err_code);

			app_pwm_disable(&PWM1);

			app_timer_stop(m_ms_timer_id);
			app_timer_stop(m_our_char_timer_id);

			
	
            app_timer_start(wakeup_timer_id, WAKEUP_TIME_INVERVAL, NULL);
            err_code = sd_app_evt_wait();
            APP_ERROR_CHECK(err_code);
#endif
        break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
        break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
        break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
        break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
        break;

        default:
            // No implementation needed.
        break;
    }
}






/**@brief Function for the SoftDevice initialization.
 *
 * @details This function initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    
    /* enable DCDC to save current */
    err_code = sd_power_dcdc_mode_set(NRF_POWER_DCDC_ENABLE);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**@brief Function for initializing the GATT library. */
void gatt_init(void)
{
    ret_code_t err_code;

    err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;

    NRF_LOG_INFO("BSP_EVENT_HANDLER => %d", event);

    switch (event)
    {

		case BSP_EVENT_DFU:	// 들어오지 않음
			NRF_LOG_INFO("@@@@@@@ BSP_EVENT_DFU=>%d", event); 
		break;

		case BSP_EVENT_ADVERTISING_START:	// 들어오지 않음
			NRF_LOG_INFO("@@@@@@@ BSP_EVENT_ADVERTISING_START=>%d", event);	
		break;


		case BSP_EVENT_ADVERTISING_STOP:	// 들어오지 않음
			NRF_LOG_INFO("@@@@@@@ BSP_EVENT_ADVERTISING_STOP=>%d", event);	
		break;

	
        case BSP_EVENT_SLEEP:	// 들어오지 않음
        	NRF_LOG_INFO("@@@@@@@ bsp_event_handler() --  BSP_EVENT_SLEEP");
            //sleep_mode_enter();
        break;

		case BSP_EVENT_WAKEUP:  // 들어오지 않음

		    NRF_LOG_INFO("@@@@@@@ BSP_EVENT_WAKEUP=>%d", event);	
			
			advertising_flag = 0;

		break;

        case BSP_EVENT_DISCONNECT:

		  	NRF_LOG_INFO("bsp_event_handler() --  BSP_EVENT_DISCONNECT");
		  
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
        break;

        case BSP_EVENT_WHITELIST_OFF:

		  	NRF_LOG_INFO("bsp_event_handler() --  BSP_EVENT_WHITELIST_OFF");

            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
		break;

		case BSP_EVENT_KEY_1:		// event 15		TEST

			LoadReceiverMac();

			Battery_Adc_Read();
		
			err_code = app_timer_start(m_ms_timer_id, TIMER_MS_INTERVAL, NULL);
			APP_ERROR_CHECK(err_code);
			err_code = app_timer_start(m_our_char_timer_id, OUR_CHAR_TIMER_INTERVAL, NULL);
			APP_ERROR_CHECK(err_code);			
			//strncpy( DEVICE_NAME, "ICSWES", 6);
		break;


		case BSP_EVENT_KEY_2:		// event 16		EMG

			LoadReceiverMac();
			//fds_register(fds_evt_handler);
			//fds_init();
			
			Battery_Adc_Read();
		
			err_code = app_timer_start(m_ms_timer_id, TIMER_MS_INTERVAL, NULL);
			APP_ERROR_CHECK(err_code);
			err_code = app_timer_start(m_our_char_timer_id, OUR_CHAR_TIMER_INTERVAL, NULL);
			APP_ERROR_CHECK(err_code);
			


		break;

        default:


		break;

    }
}





/* *@brief Undo the changes that we did when advertising has the timeout.
*/
void wakeup_timer_handler(void * p_context)
{

	uint32_t err_code;

	
	app_timer_stop(wakeup_timer_id);

	app_timer_start(m_ms_timer_id, TIMER_MS_INTERVAL, NULL);
	app_timer_start(m_our_char_timer_id, OUR_CHAR_TIMER_INTERVAL, NULL);

	bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
	
	APP_ERROR_CHECK(err_code);
	//err_code = ble_advertising_start(BLE_ADV_MODE_FAST);
	err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
	APP_ERROR_CHECK(err_code);

	SystemManager.EmgSendButton = BUTTON_ALIVE;
	NRF_LOG_INFO("==> Alive Send");

	SystemTimer.EMG_SEND_RUN = 1;
	SystemTimer.SEND_TIMER = 0;


}



static void timer_ms_timeout_handler(void * p_context)
{
	UNUSED_PARAMETER(p_context);

	HSDInRun();
	
}

	


/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to
 *          a string. The string will be be sent over BLE when the last character received was a
 *          'new line' '\n' (hex 0x0A) or if the string has reached the maximum data length.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:

			NRF_LOG_DEBUG("uart_event_handle -- APP_UART_DATA_READY");
		
            UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;

            if ((data_array[index - 1] == '\n') ||
                (data_array[index - 1] == '\r') ||
                (index >= m_ble_nus_max_data_len))
            {
                if (index > 1)
                {
                    NRF_LOG_DEBUG("Ready to send data over BLE NUS");
                    NRF_LOG_HEXDUMP_DEBUG(data_array, index);

                    do
                    {
                        uint16_t length = (uint16_t)index;
                        err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);
                        if ((err_code != NRF_ERROR_INVALID_STATE) &&
                            (err_code != NRF_ERROR_RESOURCES) &&
                            (err_code != NRF_ERROR_NOT_FOUND))
                        {
                            APP_ERROR_CHECK(err_code);
                        }
                    } while (err_code == NRF_ERROR_RESOURCES);
                }

                index = 0;
            }
            break;

        case APP_UART_COMMUNICATION_ERROR:

			NRF_LOG_DEBUG("uart_event_handle -- APP_UART_COMMUNICATION_ERROR");

            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
			
			NRF_LOG_DEBUG("uart_event_handle -- APP_UART_FIFO_ERROR");

		
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;
		
		case APP_UART_TX_EMPTY:

			NRF_LOG_DEBUG("uart_event_handle -- APP_UART_TX_EMPTY");

			break;

		case APP_UART_DATA:

			NRF_LOG_DEBUG("uart_event_handle -- APP_UART_DATA");

			break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
 */
/**@snippet [UART Initialization] */
static void uart_init(void)
{
    uint32_t                     err_code;
    app_uart_comm_params_t const comm_params =
    {
        .rx_pin_no    = RX_PIN_NUMBER,
        .tx_pin_no    = TX_PIN_NUMBER,
        .rts_pin_no   = RTS_PIN_NUMBER,
        .cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
#if defined (UART_PRESENT)
        .baud_rate    = NRF_UART_BAUDRATE_115200
#else
        .baud_rate    = NRF_UARTE_BAUDRATE_115200
#endif
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}
/**@snippet [UART Initialization] */





/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{

 //p_m_advertising->adv_data.adv_data.p_data = p_m_advertising->enc_advdata;

	ret_code_t			   err_code;
	ble_advertising_init_t init;
	char plevel = 0xfe;
	ble_advdata_conn_int_t connParams;

	connParams.min_conn_interval = 0x20;
	connParams.max_conn_interval = 0x28;

	memset(&init, 0, sizeof(init));

	init.advdata.name_type 		 = BLE_ADVDATA_FULL_NAME;
	init.advdata.include_appearance = false;
	init.advdata.flags 			 = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;
	init.advdata.p_tx_power_level = &plevel;
	init.advdata.p_slave_conn_int = &connParams;


	// Prepare the scan response manufacturer specific data packet
	ble_advdata_manuf_data_t  manuf_data_response;
	uint8_t					 data_response[] = "M";
	
	manuf_data_response.company_identifier  = 0x0059;
	manuf_data_response.data.p_data		 = data_response;
	manuf_data_response.data.size			 = sizeof(data_response);
	init.srdata.name_type = BLE_ADVDATA_FULL_NAME;
	init.srdata.p_manuf_specific_data = &manuf_data_response;

	init.config.ble_adv_fast_enabled  = true;
	init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
	init.config.ble_adv_fast_timeout  = APP_ADV_DURATION;

	init.evt_handler = on_adv_evt;

	err_code = ble_advertising_init(&m_advertising, &init);
	APP_ERROR_CHECK(err_code);

	ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
 
	p_m_advertising = &m_advertising;

}


/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
    bsp_event_t startup_event;

    uint32_t err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);

    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}


/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    if (NRF_LOG_PROCESS() == false)
    {
        nrf_pwr_mgmt_run();
    }
}


/**@brief Function for starting advertising.
 */
void advertising_start(void)
{

    uint32_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
    APP_ERROR_CHECK(err_code);

	advertising_flag = 1;
}



static void leds_config(void)
{
    bsp_board_init(BSP_INIT_LEDS);
}



void Battery_Adc_Read(void)
{
	uint32_t avr = 0;

	nrf_delay_ms(5);
	nrf_drv_saadc_sample_convert(NRF_SAADC_INPUT_AIN2, &m_buffer_pool[0]);
	nrf_delay_ms(5);
	nrf_drv_saadc_sample_convert(NRF_SAADC_INPUT_AIN2, &m_buffer_pool[1]);
	nrf_delay_ms(5);
	nrf_drv_saadc_sample_convert(NRF_SAADC_INPUT_AIN2, &m_buffer_pool[2]);

	avr = m_buffer_pool[0] + m_buffer_pool[1] + m_buffer_pool[2];
	avr /= 3;


	SystemManager.Batt = (((float)avr*0.708)-200.0)*(100.0/(330.0-200.0));

	if(SystemManager.Batt > 100)
		SystemManager.Batt = 100;

	NRF_LOG_INFO("Vol:%d -- [%d]\r\n",(int)((float)avr*0.708), SystemManager.Batt);

	

#if 0
	if(nrf_drv_saadc_sample_convert(NRF_SAADC_INPUT_AIN2, m_buffer_pool) == NRF_SUCCESS)
	{
//		SystemManager.Batt = (((float)m_buffer_pool[0]*0.708)/330.0)*100;
		SystemManager.Batt = (((float)m_buffer_pool[0]*0.708)-200.0)*(100.0/(330.0-200.0));

		if(SystemManager.Batt > 100)
			SystemManager.Batt = 100;

		NRF_LOG_INFO("Vol:%d -- [%d]\r\n",(int)((float)m_buffer_pool[0]*0.708), SystemManager.Batt);
	}
#endif


}




void Buzzer_Select(uint8_t state)
{
	switch(state){
		case BUZZER_NONE:

		break;

		case BUZZER_EMG:
			if( SystemManager.BuzzerType == BUZZER_NONE )
			{
				SystemManager.BuzzerType = 1;

				if(SystemManager.Batt > BATTERY_LOW_LEVEL)
					SystemTimer.BUZZER_TIMER = 200;
				else
					SystemTimer.BUZZER_TIMER = 600;
				
				app_pwm_enable(&PWM1);
				app_pwm_channel_duty_set(&PWM1, 0, 50);
			}
		break;

		case BUZZER_TEST:
			if( SystemManager.BuzzerType == BUZZER_NONE )
			{
				SystemManager.BuzzerType = 2;
			
				if(SystemManager.Batt > BATTERY_LOW_LEVEL)
					SystemTimer.BUZZER_TIMER = 100;
				else
					SystemTimer.BUZZER_TIMER = 600;
				app_pwm_enable(&PWM1);
				app_pwm_channel_duty_set(&PWM1, 0, 50);
			}
		break;

		case BUZZER_REG:

			if( SystemManager.BuzzerType == BUZZER_NONE )
			{
				SystemManager.BuzzerType = 3;
				SystemTimer.BUZZER_TIMER = 500;
				app_pwm_enable(&PWM1);
				app_pwm_channel_duty_set(&PWM1, 0, 50);
			}

		break;

		case BUZZER_BOOT_LOADER:

			if( SystemManager.BuzzerType == BUZZER_NONE )
			{
				SystemManager.BuzzerType = 4;
				SystemTimer.BUZZER_TIMER = 1000;
				app_pwm_enable(&PWM1);
				app_pwm_channel_duty_set(&PWM1, 0, 50);
			}

		break;		

		case BUZZER_POWER_ON:

			if( SystemManager.BuzzerType == BUZZER_NONE )
			{
				SystemManager.BuzzerType = BUZZER_POWER_ON;
				
				SystemTimer.BUZZER_TIMER = 300;
				
				
				app_pwm_enable(&PWM1);
				app_pwm_channel_duty_set(&PWM1, 0, 50);

			}

		break;	

		
	}

}


// 송신기 EMG 신호를 전송
void SendEmgData(void)
{

    static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
    static uint8_t index = 0;
    uint32_t err_code;
    uint16_t length;
	int i;


	NRF_LOG_INFO("SendEmgData(%d)", SystemManager.Batt);


    data_array[0] = COMMAND_TYPE;

    data_array[1] = 0;    
    data_array[2] = 0;
    data_array[3] = 0;
    data_array[4] = 0;
	
    data_array[5] = 1;
    data_array[6] = 1;
    data_array[7] = 1;
    data_array[8] = 1;
	
    data_array[9]  = new_ble_addr.addr[3];
    data_array[10] = new_ble_addr.addr[4];
    data_array[11] = new_ble_addr.addr[5];
	
    data_array[12] = COMMAND_EMG;
	
    data_array[13] = SystemManager.Batt;
	
    data_array[14] = VER_HIGH_VALUE;
    data_array[15] = VER_LOW_VALUE;        
    data_array[16] = 0x3;

	length = COMMAND_LENGTH;


	WesEncryptEcb( &data_array[1] );

    err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);


	for( i=0; i<17; i++)
	{
		NRF_LOG_RAW_INFO("%02x ", data_array[i]);
	}
	
	NRF_LOG_RAW_INFO("\n");



    SystemManager.BleSendStatus = err_code;

    if ((err_code != NRF_ERROR_RESOURCES)&&(err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY) && (err_code != NRF_ERROR_NOT_FOUND))// (or anything else you want to filter out...)
    {
        APP_ERROR_CHECK(err_code);
    }

	//SystemTimer.SLEEP_TIMER = 0;
	//SystemManager.SleepEnter = 1;
	
}



// 송신기 Test 신호를 전송
void SendTestData(void)
{

	static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
	static uint8_t index = 0;
	uint32_t	   err_code;
	uint16_t length;

	NRF_LOG_INFO("SendTestData()");


	data_array[0] = COMMAND_TYPE;
	
	data_array[1] = 0;	  
	data_array[2] = 0;
	data_array[3] = 0;
	data_array[4] = 0;
	
	data_array[5] = 1;
	data_array[6] = 1;
	data_array[7] = 1;
	data_array[8] = 1;
	
	data_array[9] 	= new_ble_addr.addr[3];
	data_array[10] = new_ble_addr.addr[4];
	data_array[11] = new_ble_addr.addr[5];
	
	data_array[12] = COMMAND_TEST;
	
	data_array[13] = SystemManager.Batt;
	
    data_array[14] = VER_HIGH_VALUE;
    data_array[15] = VER_LOW_VALUE;
	data_array[16] = 0x3;
	
	length = COMMAND_LENGTH;

	WesEncryptEcb( &data_array[1] );

	err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);

	SystemManager.BleSendStatus = err_code;

	if ((err_code != NRF_ERROR_RESOURCES)&&(err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY) && (err_code != NRF_ERROR_NOT_FOUND))// (or anything else you want to filter out...)
	{
		APP_ERROR_CHECK(err_code);
	}

}




void SendAliveData(void)
{

	static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
	static uint8_t index = 0;
	uint32_t	   err_code;
	uint16_t length;

	NRF_LOG_INFO("SendAliveData()");

	data_array[0] = COMMAND_TYPE;

	data_array[1] = 0;	  
	data_array[2] = 0;
	data_array[3] = 0;
	data_array[4] = 0;
	
	data_array[5] = 1;
	data_array[6] = 1;
	data_array[7] = 1;
	data_array[8] = 1;
	
	data_array[9] = new_ble_addr.addr[3];
	data_array[10] = new_ble_addr.addr[4];
	data_array[11] = new_ble_addr.addr[5];
	
	data_array[12] = COMMAND_ALIVE;
	
	data_array[13] = SystemManager.Batt;
	
    data_array[14] = VER_HIGH_VALUE;
    data_array[15] = VER_LOW_VALUE;      
	data_array[16] = 0x3;

	length = COMMAND_LENGTH;

	WesEncryptEcb( &data_array[1] );

	err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);

	SystemManager.BleSendStatus = err_code;

	if ((err_code != NRF_ERROR_RESOURCES)&&(err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY) && (err_code != NRF_ERROR_NOT_FOUND))// (or anything else you want to filter out...)
	{
		APP_ERROR_CHECK(err_code);
	}

}





// 등록모드 
void SendRegData(void)
{

	static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
	static uint8_t index = 0;
	uint32_t	   err_code;
	uint16_t length;


	NRF_LOG_INFO("SendRegData()");


#if 0
	if( SystemManager.BuzzerType == 0 )
	{
		SystemManager.BuzzerType = 3;
		SystemTimer.BUZZER_TIMER = 500;
		app_pwm_enable(&PWM1);
		app_pwm_channel_duty_set(&PWM1, 0, 50);
	}
#endif

	data_array[0] = COMMAND_TYPE;

	data_array[1] = 0;	  
	data_array[2] = 0;
	data_array[3] = 0;
	data_array[4] = 0;
	
	data_array[5] = 1;
	data_array[6] = 1;
	data_array[7] = 1;
	data_array[8] = 1;
	
	data_array[9] =  new_ble_addr.addr[3];
	data_array[10] = new_ble_addr.addr[4];
	data_array[11] = new_ble_addr.addr[5];
	
	data_array[12] = COMMAND_REG;
	
	data_array[13] = SystemManager.Batt;
	
    data_array[14] = VER_HIGH_VALUE;
	data_array[15] = VER_LOW_VALUE; 
	data_array[16] = 0x3;
	
 	length = COMMAND_LENGTH;

	WesEncryptEcb( &data_array[1] );

	err_code = ble_nus_data_send(&m_nus, data_array, &length, m_conn_handle);

	SystemManager.BleSendStatus = err_code;

	if ((err_code != NRF_ERROR_RESOURCES)&&(err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY) && (err_code != NRF_ERROR_NOT_FOUND))// (or anything else you want to filter out...)
	{
		APP_ERROR_CHECK(err_code);

		RegMacDelteAll();
		SaveReceiverMac(SystemManager.StrPeerAddr);
	}else{
		
	}

}



void pwm_ready_callback(uint32_t pwm_id)    // PWM callback function
{
    ready_flag = true;
}


void pwmInit(void)
{

    ret_code_t err_code;

   app_pwm_config_t pwm1_cfg = APP_PWM_DEFAULT_CONFIG_1CH(370L, 7);		//1000L -> 1Khz, 500L -> 2Khz,  370L -> 2.7Khz   


    /* Switch the polarity of the second channel. */
    //pwm1_cfg.pin_polarity[0] = APP_PWM_POLARITY_ACTIVE_LOW;
	pwm1_cfg.pin_polarity[0] = APP_PWM_POLARITY_ACTIVE_HIGH;
	

    /* Initialize and enable PWM. */
    err_code = app_pwm_init(&PWM1,&pwm1_cfg,pwm_ready_callback);
    APP_ERROR_CHECK(err_code);
}



#define SND_WDT_SLEEP_CONFIG                                               \
{                                                                         \
    .behaviour          = (nrf_wdt_behaviour_t)NRFX_WDT_CONFIG_BEHAVIOUR, \
    .reload_value       = SLEEP_WDT_LIMIT,                   \
    NRFX_WDT_IRQ_CONFIG                                                   \
}


void WdtRunMode()
{
	nrf_wdt_reload_value_set(RUN_TIME_LIMIT);	//10sec

}



void WdtSleepMode()
{
	uint32_t err_code;
    nrf_drv_wdt_config_t config = SND_WDT_SLEEP_CONFIG;

    err_code = nrf_drv_wdt_init(&config, wdt_event_handler);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_wdt_channel_alloc(&m_channel_id);
    APP_ERROR_CHECK(err_code);
    nrf_drv_wdt_enable();
}


void ReMac_Mapping()
{


	ble_gap_addr_t dd;
	dd.addr_id_peer = 0;

 	dd.addr_type = BLE_GAP_ADDR_TYPE_PUBLIC;

	sd_ble_gap_addr_get(&old_ble_addr);

	dd.addr[0] = IES200_1ST_MAC;
	dd.addr[1] = IES200_2ST_MAC;
	dd.addr[2] = IES200_3ST_MAC;
	dd.addr[3] = old_ble_addr.addr[3];
	dd.addr[4] = old_ble_addr.addr[4];
	dd.addr[5] = old_ble_addr.addr[5];
	
	sd_ble_gap_addr_set(&dd);
	sd_ble_gap_addr_get(&new_ble_addr);

	DEVICE_NAME[9] = dd.addr[3];
	DEVICE_NAME[10] = dd.addr[4];
	DEVICE_NAME[11] = dd.addr[5];

	NRF_LOG_INFO("MAC Address : %02X %02X %02X", dd.addr[3], dd.addr[4], dd.addr[5]);

}

/**@brief Application main function.
 */
int main(void)
{

	uint32_t err_code;

	bool erase_bonds;

	NRF_LOG_INFO("\n\n\n\n");	
	NRF_LOG_INFO("IES-200B Program Start --- Ver 0.92 2022/03/10");	

	// Initialize.
	//uart_init();
	SaadcInit();
	log_init();

	/* Register first to receive an event when initialization is complete. */
    (void) fds_register(fds_evt_handler);
	
	pwmInit();

	err_code = fds_init();
    APP_ERROR_CHECK(err_code);

	
	timers_init();
	HSDInInit();

	SystemManager.GpReg = nrf_power_gpregret_get();
	buttons_leds_init(&erase_bonds);
	power_management_init();
	ble_stack_init();

	ReMac_Mapping();

	SystemManager.ResetReason =  NRF_POWER->RESETREAS;
	SystemManager.BleSendStatus = -1;
	
	gap_params_init();
	gatt_init();
	services_init();
	advertising_init();
	conn_params_init();

	err_code = app_timer_create(&wakeup_timer_id, APP_TIMER_MODE_SINGLE_SHOT, wakeup_timer_handler);
    APP_ERROR_CHECK(err_code);
	err_code = app_timer_create(&m_ms_timer_id, APP_TIMER_MODE_REPEATED, timer_ms_timeout_handler);
    APP_ERROR_CHECK(err_code);

	//err_code = app_timer_start(m_ms_timer_id, TIMER_MS_INTERVAL, NULL);
	//APP_ERROR_CHECK(err_code);
	//err_code = app_timer_start(m_our_char_timer_id, OUR_CHAR_TIMER_INTERVAL, NULL);
	//APP_ERROR_CHECK(err_code);

	// Start execution.
	//printf("\r\nUART started.\r\n");
	NRF_LOG_INFO("Debug logging for UART over RTT started.");
	//advertising_start();

    ParkSysInit();
	bsp_board_leds_off();

	SystemManager.SendRepeat = 0;
	SystemManager.SendRepeatTimerLimit = 30000;	// on time

	err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);
	

	if( SystemManager.GpReg == 0x00 )
	{
		err_code = app_timer_start(m_ms_timer_id, TIMER_MS_INTERVAL, NULL);
		APP_ERROR_CHECK(err_code);
		err_code = app_timer_start(m_our_char_timer_id, OUR_CHAR_TIMER_INTERVAL, NULL);
		APP_ERROR_CHECK(err_code);			


		SystemManager.EmgSendButton = BUTTON_EMG;
		SystemTimer.EMG_SEND_RUN = 1;
		SystemTimer.SEND_TIMER = 0;	
		Buzzer_Select(BUZZER_POWER_ON);
		
		//nrf_power_gpregret_set(1);
		for (;;)
		{
			idle_state_handle();

			if( SystemManager.GoSleep == 1 )
			{
				app_timer_stop(m_ms_timer_id);
				app_timer_stop(m_our_char_timer_id);
				break;
			}
		}
		nrf_power_gpregret_set(2);
	}
	else if( SystemManager.GpReg == 0x04  )
	{
		err_code = app_timer_start(m_ms_timer_id, TIMER_MS_INTERVAL, NULL);
		APP_ERROR_CHECK(err_code);
		err_code = app_timer_start(m_our_char_timer_id, OUR_CHAR_TIMER_INTERVAL, NULL);
		APP_ERROR_CHECK(err_code);			

		SystemManager.EmgSendButton = BUTTON_REG;
		SystemTimer.EMG_SEND_RUN = 1;
		SystemTimer.SEND_TIMER = 0;	
		Buzzer_Select(BUZZER_POWER_ON);
		
		//nrf_power_gpregret_set(1);
		for (;;)
		{
			idle_state_handle();

			if( SystemManager.GoSleep == 1 )
			{
				app_timer_stop(m_ms_timer_id);
				app_timer_stop(m_our_char_timer_id);
				break;
			}
		}
		nrf_power_gpregret_set(2);
	}
	

	WdtSleepMode();


    app_timer_start(wakeup_timer_id, WAKEUP_TIME_INVERVAL, NULL);		// Wake Up Timer Reset
    err_code = sd_app_evt_wait();										// jakuja  GOTO SLEEP MODE
    APP_ERROR_CHECK(err_code);


	
	Battery_Adc_Read();

	WdtRunMode();



    //Configure WDT.
    //nrf_drv_wdt_config_t config = NRF_DRV_WDT_DEAFULT_CONFIG;
    //err_code = nrf_drv_wdt_init(&config, wdt_event_handler);
    //APP_ERROR_CHECK(err_code);
    //err_code = nrf_drv_wdt_channel_alloc(&m_channel_id);
    //APP_ERROR_CHECK(err_code);
    //nrf_drv_wdt_enable();


	// Enter main loop.
	for (;;)
	{
		SysCounter++;
		idle_state_handle();
	}
}


/**
 * @}
 */
